// 
// Dateiname:    testLR.cc (Testprogramm für die LR-Zerlegung)
// Kompilieren:  g++ -I../hdnum -o testLR testLR.cc
//
#include <iostream>
#include <iomanip>
#include <cstdlib>

#include "hdnum.hh"

#include "solveR.hh"
#include "solveL.hh"
#include "LR.hh"

typedef double number;
//typedef float number;

int main(int argc, char ** argv){
  
  if(argc < 2){
    std::cout << "Usage  : ./testLR Dimension" << std::endl;
    std::cout << "Example: ./testLR 7" << std::endl;
    exit(0);
  }
  else{
    int n = atoi( argv[1] );
    std::cout << "Dimension=" << n << std::endl;

    hdnum::DenseMatrix<number> A(n,n);
    hdnum::Vector<number> x(n);
    hdnum::Vector<number> b(n);

    
    // Testvektor x:
    hdnum::fill(x,number(1.0),number(1.0));   // x = (1,2,3,4,...)
    std::cout << "x=" << x << std::endl;

    // Testmatrix A:
    hdnum::vandermonde(A,x);
    std::cout << "A=" << A << std::endl;

    // Testvektor b:
    A.mv(b,x);           // b = A*x
    std::cout << "b=A*x=" << b << std::endl;

    hdnum::Vector<number> y(n); // fuer die Loesung
    hdnum::Vector<number> z(n); // fuer die Zwischenloesung

    // Hilfsvektoren:
    hdnum::Vector<number> s(n);      // speichert die Zeilensummen von A
    hdnum::Vector<std::size_t> p(n); // Indexvektor, speichert die Zeilenvertauschungen

    lr(A,p,true);
    permute_forward(p,b,true);  // Wende Zeilenvertauschungen auf b an.
    solveL(A,z,b);         // Loese die Gleichung L*z = b
    solveR(A,y,z);         // Loese die Gleichung R*y = z, d.h. zusammen: L*R*y = b

    
    // Vergleiche Loesungsvektor y mit der echten Loesung x.
    std::cout << "y=" << y << std::endl;

    std::cout << "||y-x||=" << (y-x).two_norm() << std::endl;
    
    return 0;
  }
}
